#pragma once

#include <map>
#include <functional>

#include "hack/exception/exception.hpp"

namespace trs
{
  template<typename Transaction>
  class function_manager
  {
    using functions = std::map<const std::string, std::function<void(Transaction&)>>;

    public:
      function_manager() = default;
      ~function_manager() = default;

    private:
      functions m_functions;

    public:
      void valid(std::string func_name) const
      { 
        if (m_functions.count(func_name) == 0)
        {
          hack::exception ex;
          ex.description("this function is not register in function manager");
          throw ex; 
        }
      }
      
      template<typename Key, typename Function>
      void registration(const Key k, const Function f) noexcept
      {
        m_functions[k] = f;
      }

      void execute(Transaction& tr) 
      {
        m_functions[tr.m_passport.m_function_name](tr);
      }
  }; 
}
