#pragma once

#include <string>
#include "hack/exception/exception.hpp"

namespace trs
{
  template<typename Transaction>
  class inspector
  {
    using functions = std::map<const std::string, std::function<void(Transaction&)>>;

    public:
      inspector() = default;
      ~inspector() = default;

    public:
      template<typename Key, typename Function>
      void registration(const Key k, const Function f) noexcept
      {
        m_functions[k] = f;
      }

      void valid(Transaction& tr) 
      {
        if (!valid(tr.m_passport.m_function_name))
        {
          hack::exception ex;
          ex.description("Function is not register in inspector. Please do it");
          throw ex; 
        }

        m_functions[tr.m_passport.m_function_name](tr);
      }

    private:
      bool valid(std::string func_name) const noexcept
      { 
        return m_functions.count(func_name) > 0; 
      }

    private:
      functions m_functions;
  };
}
