#pragma once

#include <utility>
#include <string>

#include "hack/exception/exception.hpp"
#include "hack/string/string_concat_helper.hpp"

#include "using.hpp"

namespace trs::helpers
{
  namespace
  {
    inline std::pair<bool, std::string> json_compare_impl(JSON& target, JSON& comp)
    {
      if (target.empty())
        return { false, "payload is empty" };

      for (auto&& [key, comp_v] : comp.items())
      {
        if (comp_v.is_object()) 
        {
          auto [ok, k] = json_compare_impl(target[key], comp_v);
          if (!ok) return { ok, k };
        }

        if (target[key].type_name() != comp_v.type_name())
          return { false, hack::string::str_concat + "field - [" + key + "] is invalid."};
      }

      if (target.size() != comp.size()) return { false, "payload not competed" };

      return { true, "" };
    }
  }

  template<typename Transaction>
  inline void json_compare(Transaction& base, JSON& comp)
  {
    auto [ok, msg] = json_compare_impl(base.m_data.m_payload, comp);
    if (!ok) 
    { 
      hack::exception ex;
      ex.description(msg); 
      throw ex; 
    }
  }
}
